# -*- coding: utf-8 -*-
"""
Created on Thu Sep  8 11:00:11 2016

@author: david
"""
import os
import ete2
import glob
import argparse
from collections import Counter

import tree
import root_from_duplications as rfd     
  
nProcs = 8     
     
def PlotTree(speciesTreeFN, supported_clusters, outputFN):
    c = Counter(supported_clusters)
    S = set(speciesTree.get_leaf_names())
    tree_for_figure = ete2.Tree(speciesTreeFN)
    for cluster, count in c.items():
#        print(count)
#        print(cluster)
        complement = None
        if len(cluster) == 1: continue
        n = tree_for_figure.get_common_ancestor(cluster)
        # cluster could straddle root
        if n == tree_for_figure:
            complement = S.difference(cluster) # complement
#            print(complement)
            if len(complement) == 1:
#                continue
                n = tree_for_figure & list(complement)[0]
            else:
                n = tree_for_figure.get_common_ancestor(complement)
#        print(('r' if complement != None else 'g', count, cluster))
        face = ete2.faces.TextFace(" " + str(count), fgcolor=('red' if complement else 'green'))
        n.add_face(face, 1)
    print(outputFigFN)
    tree_for_figure.render(outputFigFN)
        
        
if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("input_tree")
    parser.add_argument("-s", "--separator", choices=("dot", "dash", "second_dash", "third_dash"))
    parser.add_argument("-S", "--Species_tree")
    parser.add_argument("-d", "--directory", action="store_true")
    parser.add_argument("--debug", action="store_true", help="Run in serial to enable easier debugging")
    args = parser.parse_args()
    spTreeFormat = 1
    
    GeneToSpecies = rfd.GeneToSpecies_dash
    if args.separator and args.separator == "dot":
        GeneToSpecies = rfd.GeneToSpecies_dot  
    elif args.separator and args.separator == "second_dash":
        GeneToSpecies = rfd.GeneToSpecies_secondDash  
    elif args.separator and args.separator == "3rd_dash":
        GeneToSpecies = rfd.GeneToSpecies_3rdDash  
    
    if not args.directory:
        speciesTree = tree.Tree(args.Species_tree, format=spTreeFormat)
        species, dict_clades, clade_names = rfd.AnalyseSpeciesTree(speciesTree)
        t = tree.Tree(args.input_tree, format=1)
        G = set(t.get_leaf_names())
        S = list(set(map(GeneToSpecies, G)))
        filename = 'profile_stats.stats'
#        profile.run('c = SupportedHierachies_wrapper(args.input_tree, GeneToSpecies, species, dict_clades, clade_names) ', filename)
        c = rfd.SupportedHierachies_wrapper(args.input_tree, GeneToSpecies, species, dict_clades, clade_names)      
        for cc in c: print(cc)
#        stats = pstats.Stats('profile_stats.stats')
#        stats.sort_stats('cumulative')
#        stats.print_stats(0.3)
    elif args.debug:
        speciesTree = tree.Tree(args.Species_tree, format=spTreeFormat)
        species, dict_clades, clade_names = rfd.AnalyseSpeciesTree(speciesTree)
        clusters = []
        for fn in glob.glob(args.input_tree + "/*"):
            c = rfd.SupportedHierachies_wrapper(fn, GeneToSpecies, species, dict_clades, clade_names)
            clusters.extend(c)
        roots, nSupport = rfd.ParsimonyRoot(species, dict_clades.keys(), clusters)
        if len(roots) > 1: print("Observed %d duplications. %d support the best roots and %d contradict them." % (len(clusters), nSupport, len(clusters) - nSupport))
        else: print("Observed %d duplications. %d support the best root and %d contradict it." % (len(clusters), nSupport, len(clusters) - nSupport))
        print("Best outgroup(s) for species tree:")
        for r in roots: 
            print(r)
    else:
        print("%d trees" % len(glob.glob(args.input_tree + "/*")))
        roots, clusters, _, nSupport = rfd.GetRoot(args.Species_tree, args.input_tree, GeneToSpecies, nProcs, treeFmt = 1)
        if len(roots) > 1: print("Observed %d duplications. %d support the best roots and %d contradict them." % (len(clusters), nSupport, len(clusters) - nSupport))
        else: print("Observed %d duplications. %d support the best root and %d contradict it." % (len(clusters), nSupport, len(clusters) - nSupport))
        print("Best outgroup(s) for species tree:")
        for r in roots: 
            print(r)
        speciesTree = tree.Tree(args.Species_tree, format=1)
        outputFigFN = args.input_tree + "/../Duplications_%s.pdf" % os.path.split(args.input_tree)[0].split("/")[-1] 
        PlotTree(args.Species_tree, clusters, outputFigFN)